/* gfxtest.c
 *
 * A little program to test out various graphics modes
 *
 */

#include <sys/file.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <amiga/dev/grfioctl.h>

#include <stdio.h>
#include <stdlib.h>

static unsigned short rgb2short(int, int, int, int);

int
main(ac,av)
int ac;
char **av;
{
    int grffd, x, y, z;
    int yes = 1, no = 0;
    int vmode = 1;
    struct grfvideo_mode gv;
    unsigned char *fb;
    volatile unsigned char *ba;
    char dummy[10];

    if (ac > 1)
    	vmode = atoi(av[1]);

    grffd = open("/dev/grf3", O_RDWR);
    if (grffd < 0)
    {
    	perror("gfxtest");
    	return(0);
    }
    
    fb = mmap(0, 1024*1024*2, PROT_READ|PROT_WRITE, 0, grffd, 65536);
    if ((int)fb < 0)
    {
    	perror("gfxtest mmap");
    	return(0);
    }
    ba = mmap(0, 65536, PROT_READ|PROT_WRITE, 0, grffd, 0);

    ioctl(grffd, GRFSETVMODE, &vmode);
    ioctl(grffd, GRFIOCON, &yes);
    gv.mode_num = vmode;
    ioctl(grffd, GRFGETVMODE, &gv);

#ifdef 0
    *(ba + 0x3c6) = 0xff;
    *(ba + 0x3c6);
    *(ba + 0x3c6);
    *(ba + 0x3c6);
    *(ba + 0x3c6);
    *(ba + 0x3c6) = 0xd0;
#endif
    
    
    switch (gv.depth)
    {
    case 1:
    case 8:
    	for (x = 0; x < (gv.disp_height * gv.disp_width); x += 4)
    	    	*((unsigned long *)(fb+x)) = 0x12345678;
    	break;
    case 15:
    case 16:
    	for (x = 0; x < (gv.disp_height * gv.disp_width * 2) ; x+=2)
    	{
        	z = (x/(gv.disp_width*4)) & 0x3f;
         	y = (x/(gv.disp_width*2/4)) % 4;
    	    	
    	    	*((unsigned short *)(fb+x)) = rgb2short(gv.depth, y==1?z:0, y==2?z:0, y==3?z:0);
    	}
    	break;
    	    	
    case 24:
        for (x = 0; x < (gv.disp_height * gv.disp_width * 3) ; x+=3)
        {

        	*(fb+x+0) = y==1?z:0;
        	*(fb+x+1) = y==2?z:0;
        	*(fb+x+2) = y==3?z:0;
        }
    default:
    	break;

    }


    fgets(dummy, 4, stdin);

    munmap(fb, 1024*1024*2);
    munmap(ba, 65536);

    ioctl(grffd, GRFIOCOFF, &no);
    close(grffd);


    return(0);
}

unsigned short
rgb2short(int dep, int r, int g, int b)
{
    int x;
    short rgb;

    rgb = 0;


/* Organization of 15/8 mode:
 *
 * (when FG == 0)
 * g2g1g0r4 r3r2r1r0 FGb4b3b2 b1b0g4g3
 *
 * (when FG == 1)
 * c2c1c0xx xxxxxxxx FGxxxxc7 c6c5c4c3
 *
 * 16 bit mode:
 *
 * g2g1g0r4 r3r2r1r0 b5b4b3b2 b1b0g4g3
 *
 */
    if (dep == 15) 
    {
    	rgb |= (r&0x1f) << 8;
    	rgb |= (b&0x1f) << 2;
    	rgb |= (g&0x07) << 13;
    	rgb |= (g&0x18) >> 3;
    }
    else
    {
    	rgb |= (r&0x1f) << 8;
    	rgb |= (b&0x1f) << 3;
        rgb |= (g&0x07) << 13;
        rgb |= (g&0x38) >> 3;
    }

    return(rgb);
}
